from btns_menus.builds.abc import *

import discord
from typing import *
from discord import ui, ButtonStyle


class SButton:
    def __init__(self,
                 *,
                 label: str,
                 custom_id: Optional[str] = None,
                 disabled: bool = False,
                 style: ButtonStyle = ButtonStyle.secondary,
                 url: Optional[str] = None,
                 emoji: Optional[Union[str, discord.Emoji, discord.PartialEmoji]] = None,
                 row: Optional[int] = None,
                 response: Optional[Union[str, discord.Embed]] = None,
                 rewrite: bool = False,
                 ephemeral: bool = False,
                 delete_msg: bool = False,
                 hidden: bool = False,
                 author: discord.Member = None,
                 verify_: bool = True):
        """
        It is a decorator used to create a **Button** overwriting ui.Button

        :param label: Label of the Button
        :param custom_id: Unique ID of the Button
        :param disabled: It is used to enable/disable the Button, i.e. Preventing user from using it
        :param style: Color of the Button
        :param url: Onclick Redirects to the given url
        :param emoji: Emoji for the Button
        :param row: Places the Button in given Row
        :param response: Sends the message (str, embed) in user channel
        :param rewrite: It is used to send the message by editing the original message rather than sending a new one
        :param ephemeral: It is used to send the message where it's only visible to interacted user or to all
        :param delete_msg: Deletes the original message
        :param hidden: It hides the Button from View
        :param author: Interaction User
        :param verify_: It is used to make the func to check for author parameter or not

        :returns: Button
        """

        self.kwargs = {
            "author": author,
            "label": label,
            "custom_id": custom_id,
            "disabled": disabled,
            "style": style,
            "url": url,
            "emoji": emoji,
            "row": row,
            "response": response,
            "rewrite": rewrite,
            "ephemeral": ephemeral,
            "delete_msg": delete_msg,
            "hidden": hidden,
            "verify": verify_,
            "func": None,
            "coro_func": None
        }

        self.after_: Optional[dict] = None
        self.interaction: Optional[discord.Interaction] = None

    def update_one(self, details: Any, option: str):
        """
        Updates the option of the **Button**

        :param details: Takes any datatype for updating
        :param option: The option which should be overwritten

        :returns: None
        """

        if option not in self.kwargs.keys():
            raise ValueError(f"Invalid option `--{option}`")
        else:
            self.kwargs[option] = details

    def update(self, **options):
        """
        Updates the options of the **Button*

        :param options: takes variables

        :returns: None
        """

        for key in options:
            if key not in self.kwargs.keys():
                raise ValueError(f"Invalid option `--{key}`")
            else:
                self.kwargs[key] = options[key]

    @property
    def args(self) -> Dict:
        """
        It's a property used to get kwargs of the button

        - Aliases: ['args', 'kwargs']

        :returns: Dict
        """

        return self.kwargs

    @property
    def author(self) -> Optional[discord.Member]:
        """
        It's a property used to get author of the button

        :returns: user: discord.Member (or) None
        """

        return self.kwargs['author']

    @property
    def name(self) -> str:
        """
        It's a property used to get label of the button

        :returns: name: str
        """

        return self.kwargs['label']

    @property
    def id(self) -> Optional[str]:
        """
        It's a property used to get ID of the button

        :returns: custom_id: str (or) None
        """

        return self.kwargs['custom_id']

    @property
    def is_ephemeral(self) -> bool:
        """
        It's a property used to check whether it's ephemeral or not

        :returns: ephemeral: bool , i.e [True, False]
        """

        return self.kwargs['ephemeral']

    @property
    def hidden(self) -> bool:
        """
        It's a property used to get hidden parm of the button

        :returns: hidden: bool, i.e [True, False]
        """

        return self.kwargs['hidden']

    def after_response(self, **options):
        """
        It's an event type function which changes the provided options after button ( onclick )

        :param options: takes variables
        :returns: None
        """

        kwargs = {}
        if len(options) >= 1:
            for key in options:
                if key not in self.kwargs.keys():
                    raise ValueError(f"Invalid option `--{key}`")
                else:
                    kwargs.update({key: options[key]})

            self.after_ = kwargs

    async def add_coro_func(self, function, *args, **kwargs):
        """
        It's an asynchronous function which stores same function type
        and adds the func to button for execution after getting clicked

        :param function: takes asynchronous function
        :param args: takes args of the function provided by a user
        :param kwargs: takes kwargs of the function provided by a user
        :returns: None
        """

        self.kwargs['coro_func'] = lambda: function(*args, **kwargs)

    def add_func(self, function, *args, **kwargs):
        """
        It's a function which stores same function type and adds the func to button for execution after getting clicked

        :param function: takes a function
        :param args: takes args of the function provided by a user
        :param kwargs: takes kwargs of the function provided by a user
        :returns: None
        """

        self.kwargs['func'] = lambda: function(*args, **kwargs)

    @property
    def after_resp(self) -> Optional[Dict]:
        """
        It's a property used to get a dictionary of options which are to be changed after button click

        :returns: Dict (or) None
        """

        return self.after_


class Btn(ui.Button):
    def __init__(self, root: ClassVar, button: SButton):
        self.root = root
        self.btn = button
        self.btn_args = self.btn.args

        super().__init__(
            label=self.btn_args['label'], custom_id=self.btn_args['custom_id'],
            disabled=self.btn_args['disabled'],
            style=self.btn_args['style'], url=self.btn_args['url'], emoji=self.btn_args['emoji'],
            row=self.btn_args['row']
        )

    async def callback(self, interaction: discord.Interaction):
        self.btn.interaction = interaction
        checked = check_for_Invoker(self.btn, interaction)
        if checked:
            if self.btn_args['coro_func'] is not None:
                func = self.btn_args['coro_func']
                await func()

            if self.btn_args['func'] is not None:
                func = self.btn_args['func']
                func()

            if self.btn_args['response'] is None:
                resp = f"Button ' {self.btn.name} ' has been clicked !"
            else:
                resp = self.btn_args['response']

            if self.btn.after_resp is not None:
                for key in self.btn.after_resp:
                    self.btn.update_one(self.btn.after_resp[key], key)

            if self.btn_args['delete_msg']:
                return await interaction.message.delete()

            emph_ = self.btn.is_ephemeral
            btn_ = self.root()
            view_ = btn_.view()
            if self.btn_args['rewrite']:
                if is_embed(resp):
                    await interaction.message.edit(content="", embed=resp, view=view_)
                else:
                    await interaction.message.edit(content=resp, embed=None, view=view_)
            else:
                await interaction.message.edit(view=view_)
                if is_embed(resp):
                    await interaction.response.send_message(content="", embed=resp, ephemeral=emph_)
                else:
                    await interaction.response.send_message(content=resp, ephemeral=emph_)



from btns_menus.builds.abc import *
from btns_menus.builds.button_ import SButton, Btn
from btns_menus.builds.menu_ import SDropMenu, Menu

from typing import *
import discord
from discord import ui


class BtnAndDropMenu:
    def __init__(self, author: discord.Member, button1: SButton, menu1: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.menu1 = menu1

        self.components = [self.btn1, self.menu1]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: BtnAndDropMenu(
            self.author, self.btn1, self.menu1, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class Btn2AndDropMenu:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton, menu1: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param button2: takes button type SButton
        :param menu1: takes Dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.btn2 = button2
        self.menu1 = menu1

        self.components = [self.btn1, self.btn2, self.menu1]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: Btn2AndDropMenu(
            self.author, self.btn1, self.btn2, self.menu1, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class Btn3AndDropMenu:
    def __init__(self, author: discord.Member, button1: SButton,
                 button2: SButton, button3: SButton, menu1: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param button2: takes button type SButton
        :param button3: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.btn2 = button2
        self.btn3 = button3
        self.menu1 = menu1

        self.components = [self.btn1, self.btn2, self.btn3, self.menu1]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: Btn3AndDropMenu(self.author, self.btn1, self.btn2,
                                             self.btn3, self.menu1, timeout=self.timeout)

    def view(self) -> ui.View:
        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class Btn4AndDropMenu:
    def __init__(self, author: discord.Member, button1: SButton,
                 button2: SButton, button3: SButton, button4: SButton, menu1: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param button2: takes button type SButton
        :param button3: takes button type SButton
        :param button4: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.btn2 = button2
        self.btn3 = button3
        self.btn4 = button4
        self.menu1 = menu1

        self.components = [self.btn1, self.btn2,
                           self.btn3, self.btn4, self.menu1]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: Btn4AndDropMenu(self.author, self.btn1, self.btn2,
                                             self.btn3, self.btn4, self.menu1, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class BtnAnd2DropMenu:
    def __init__(self, author: discord.Member, button1: SButton, menu1: SDropMenu, menu2: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param menu2: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.menu1 = menu1
        self.menu2 = menu2

        self.components = [self.btn1, self.menu1, menu2]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: BtnAnd2DropMenu(
            self.author, self.btn1, self.menu1, self.menu2, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class Btn2And2DropMenu:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton, menu1: SDropMenu, menu2: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param button2: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param menu2: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.btn2 = button2
        self.menu1 = menu1
        self.menu2 = menu2

        self.components = [self.btn1, self.btn2, self.menu1, self.menu2]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: Btn2And2DropMenu(self.author, self.btn1, self.btn2, self.menu1,
                                              self.menu2, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class Btn3And2DropMenu:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton, button3: SButton,
                 menu1: SDropMenu, menu2: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param button2: takes button type SButton
        :param button3: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param menu2: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.btn2 = button2
        self.btn3 = button3
        self.menu1 = menu1
        self.menu2 = menu2

        self.components = [self.btn1, self.btn2,
                           self.btn3, self.menu1, self.menu2]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: Btn3And2DropMenu(self.author, self.btn1, self.btn2, self.btn3, self.menu1,
                                              self.menu2, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class BtnAnd3DropMenu:
    def __init__(self, author: discord.Member, button1: SButton,
                 menu1: SDropMenu, menu2: SDropMenu, menu3: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param menu2: takes dropmenu type SDropMenu
        :param menu3: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.menu1 = menu1
        self.menu2 = menu2
        self.menu3 = menu3

        self.components = [self.btn1, self.menu1, self.menu2, self.menu3]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: BtnAnd3DropMenu(self.author, self.btn1, self.menu1, self.menu2,
                                             self.menu3, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class Btn2And3DropMenu:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton,
                 menu1: SDropMenu, menu2: SDropMenu, menu3: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param button2: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param menu2: takes dropmenu type SDropMenu
        :param menu3: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.btn2 = button2
        self.menu1 = menu1
        self.menu2 = menu2
        self.menu3 = menu3

        self.components = [self.btn1, self.btn2,
                           self.menu1, self.menu2, self.menu3]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: BtnAnd3DropMenu(self.author, self.btn1, self.menu1, self.menu2,
                                             self.menu3, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class BtnAnd4DropMenu:
    def __init__(self, author: discord.Member, button1: SButton,
                 menu1: SDropMenu, menu2: SDropMenu, menu3: SDropMenu, menu4: SDropMenu,
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param button1: takes button type SButton
        :param menu1: takes dropmenu type SDropMenu
        :param menu2: takes dropmenu type SDropMenu
        :param menu3: takes dropmenu type SDropMenu
        :param menu4: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1
        self.menu1 = menu1
        self.menu2 = menu2
        self.menu3 = menu3
        self.menu4 = menu4

        self.components = [self.btn1, self.menu1,
                           self.menu2, self.menu3, self.menu4]
        for component_ in self.components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: BtnAnd4DropMenu(self.author, self.btn1, self.menu1, self.menu2,
                                             self.menu3, self.menu4, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component_ in self.components:
            if not component_.hidden:
                if isinstance(component_, SButton):
                    view_.add_item(Btn(self.root_, component_))
                else:
                    view_.add_item(Menu(self.root_, component_))

        return view_


class MultiBtnAndDropMenu:
    def __init__(self, author: discord.Member, buttons: List[SButton], menus: List[SDropMenu],
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param buttons: takes List of button type SButton
        :param menus: takes List of dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout
        self.btns: List = buttons
        self.menus: List = menus

        components: List = self.btns + self.menus
        for component_ in components:
            if component_.author is None:
                component_.update(author=self.author)

        self.root_ = lambda: MultiBtnAndDropMenu(
            self.author, self.btns, self.menus, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for btn_ in self.btns:
            if not btn_.hidden:
                view_.add_item(Btn(self.root_, btn_))

        for menu_ in self.menus:
            if not menu_.hidden:
                view_.add_item(Menu(self.root_, menu_))

        return view_


MultiBtnAndMenu = MultiBtnAndDropMenu


class MultiBtnsAndMenus:
    def __init__(self, author: discord.Member, components: List[Union[SButton, SDropMenu]],
                 *, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        🌟 Full Flexibility

        Responsive Buttons and DropMenus

        :param author: Interaction User
        :param components: Takes List of components (both buttons and drop menus)
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout
        self.components = components

        for component in self.components:
            if component.author is None:
                component.update(author=self.author)

        self.root_ = lambda: MultiBtnsAndMenus(
            self.author, self.components, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for component in self.components:
            if not component.hidden:
                if isinstance(component, SButton):
                    view_.add_item(Btn(self.root_, component))
                elif isinstance(component, SDropMenu):
                    view_.add_item(Menu(self.root_, component))
                else:
                    raise ValueError("The given components are Invalid !")

        return view_



from btns_menus.builds.menu_ import Menu, SDropMenu, DEFAULT_TIMEOUT

from typing import *
import discord
from discord import ui


class SingleDropMenu:
    def __init__(self, author: discord.Member, drop_menu1: SDropMenu, /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive DropMenu

        :param author: Interaction User
        :param drop_menu1: Takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.menu1 = drop_menu1

        if self.menu1.author is None:
            self.menu1.update_one(self.author, "author")

        self.root_ = lambda: SingleDropMenu(
            self.author, self.menu1, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        if not self.menu1.hidden:
            view_.add_item(Menu(self.root_, self.menu1))

        return view_


class DuoDropMenu:
    def __init__(self, author: discord.Member, drop_menu1: SDropMenu, drop_menu2: SDropMenu,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive DropMenus

        :param author: Interaction User
        :param drop_menu1: Takes dropmenu type SDropmenu
        :param drop_menu2: Takes dropmenu type SDropmenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.menus = [drop_menu1, drop_menu2]
        for menu_ in self.menus:
            if menu_.author is None:
                menu_.update_one(self.author, "author")

        self.root_ = lambda: DuoDropMenu(
            self.author, self.menus[0], self.menus[1], timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for menu_ in self.menus:
            if not menu_.hidden:
                view_.add_item(Menu(self.root_, menu_))

        return view_


class TrioDropMenu:
    def __init__(self, author: discord.Member, drop_menu1: SDropMenu, drop_menu2: SDropMenu, drop_menu3: SDropMenu,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive DropMenus

        :param author: Interaction User
        :param drop_menu1: takes dropmenu type SDropMenu
        :param drop_menu2: takes dropmenu type SDropMenu
        :param drop_menu3: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.menus = [drop_menu1, drop_menu2, drop_menu3]
        for menu_ in self.menus:
            if menu_.author is None:
                menu_.update_one(self.author, "author")

        self.root_ = lambda: TrioDropMenu(self.author, self.menus[0], self.menus[1], self.menus[2],
                                          timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for menu_ in self.menus:
            if not menu_.hidden:
                view_.add_item(Menu(self.root_, menu_))

        return view_


class QuartetDropMenu:
    def __init__(self, author: discord.Member, drop_menu1: SDropMenu, drop_menu2: SDropMenu,
                 drop_menu3: SDropMenu, drop_menu4: SDropMenu,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive DropMenus

        :param author: Interaction User
        :param drop_menu1: takes dropmenu type SDropMenu
        :param drop_menu2: takes dropmenu type SDropMenu
        :param drop_menu3: takes dropmenu type SDropMenu
        :param drop_menu4: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.menus = [drop_menu1, drop_menu2, drop_menu3, drop_menu4]
        for menu_ in self.menus:
            if menu_.author is None:
                menu_.update_one(self.author, "author")

        self.root_ = lambda: QuartetDropMenu(self.author, self.menus[0], self.menus[1], self.menus[2], self.menus[3],
                                             timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for menu_ in self.menus:
            if not menu_.hidden:
                view_.add_item(Menu(self.root_, menu_))

        return view_


class QuintetDropMenu:
    def __init__(self, author: discord.Member, drop_menu1: SDropMenu, drop_menu2: SDropMenu,
                 drop_menu3: SDropMenu, drop_menu4: SDropMenu, drop_menu5: SDropMenu,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive DropMenus

        :param author: Interaction User
        :param drop_menu1: takes dropmenu type SDropMenu
        :param drop_menu2: takes dropmenu type SDropMenu
        :param drop_menu3: takes dropmenu type SDropMenu
        :param drop_menu4: takes dropmenu type SDropMenu
        :param drop_menu5: takes dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.menus = [drop_menu1, drop_menu2,
                      drop_menu3, drop_menu4, drop_menu5]
        for menu_ in self.menus:
            if menu_.author is None:
                menu_.update_one(self.author, "author")

        self.root_ = lambda: QuintetDropMenu(self.author, self.menus[0], self.menus[1], self.menus[2],
                                             self.menus[3], self.menus[4], timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for menu_ in self.menus:
            if not menu_.hidden:
                view_.add_item(Menu(self.root_, menu_))

        return view_


class MultiDropMenu:
    def __init__(self, author: discord.Member, menus: List[SDropMenu], /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Multi DropMenus

        :param author: Interaction User
        :param menus: takes List of dropmenu type SDropMenu
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout
        self.menus = menus

        for menu_ in self.menus:
            if menu_.author is None:
                menu_.update(author=self.author)

        self.root_ = lambda: MultiDropMenu(
            self.author, self.menus, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for menu_ in self.menus:
            if not menu_.hidden:
                view_.add_item(Menu(self.root_, menu_))

        return view_



from btns_menus.builds.abc import *

import discord
from typing import *
from discord import ui
from discord.components import SelectOption


class SDropMenu:
    def __init__(self, *,
                 custom_id: str = MISSING,
                 placeholder: Optional[str] = None,
                 min_values: int = 1,
                 max_values: int = 1,
                 options: List[SelectOption] = None,
                 disabled: bool = False,
                 row: Optional[int] = None,
                 response: Optional[Union[str, discord.Embed]] = None,
                 rewrite: bool = False,
                 ephemeral: bool = False,
                 hidden: bool = False,
                 author: discord.Member = None,
                 verify_: bool = True
                 ):
        """
        It is a decorator used to create a **DropMenu** overwriting ui.Select

        :param custom_id: Unique ID of the Button
        :param placeholder: A short placeholder for DropMenu
        :param min_values: Limiting the user to select atleast minimum options in DropMenu
        :param max_values: Limiting the user to select atmost maximum options in DropMenu
        :param options: Options which are shown in DropMenu and can be selected by interacted user
        :param disabled: It is used to enable/disable the DropMenu, i.e. Preventing user from using it
        :param row: Places the DropMenu in given Row
        :param response: Sends the message (str, embed) in user channel
        :param rewrite: It is used to send the message by editing the original message rather than sending a new one
        :param ephemeral: It is used to send the message where it's only visible to interacted user or to all
        :param hidden:  It hides the Button from View
        :param author: Interaction User
        :param verify_: It is used to make the func to check for author parameter or not

        :returns: DropMenu
        """

        self.kwargs = {
            "author": author,
            "custom_id": custom_id,
            "placeholder": placeholder,
            "min_values": min_values,
            "max_values": max_values,
            "options": options,
            "disabled": disabled,
            "row": row,
            "response": response,
            "rewrite": rewrite,
            "ephemeral": ephemeral,
            "hidden": hidden,
            "verify": verify_,
            "queries": [],
            "func": None,
            "coro_func": None
        }

        self.after_: Optional[dict] = None
        self.selected_values: Optional[str, List] = None
        self.interaction: Optional[discord.Interaction] = None

    def update_one(self, details, option: str):
        if option not in self.kwargs.keys():
            raise ValueError(f"Invalid option `--{option}`")
        else:
            self.kwargs[option] = details

    def update(self, **options):
        for key in options:
            if key not in self.kwargs.keys():
                raise ValueError(f"Invalid option `--{key}`")
            else:
                self.kwargs[key] = options[key]

    @property
    def args(self) -> Dict:
        """
        It's a property used to get kwargs of the DropMenu

        - Aliases: ['args', 'kwargs']

        :returns: Dict
        """

        return self.kwargs

    @property
    def author(self) -> Optional[discord.Member]:
        """
        It's a property used to get author of the DropMenu

        :returns: user: discord.Member (or) None
        """

        return self.kwargs['author']

    @property
    def placeholder(self) -> Optional[str]:
        """
        It's a property used to get placeholder of the DropMenu

        :returns:
        """

        return self.kwargs['placeholder']

    @property
    def id(self) -> Optional[str]:
        """
        It's a property used to get ID of the DropMenu

        :returns: custom_id: str (or) None
        """

        return self.kwargs['custom_id']

    @property
    def is_ephemeral(self) -> bool:
        """
        It's a property used to check whether it's ephemeral or not

        :returns: ephemeral: bool , i.e [True, False]
        """

        return self.kwargs['ephemeral']

    @property
    def hidden(self) -> bool:
        """
        It's a property used to get hidden parm of the DropMenu

        :returns: hidden: bool, i.e [True, False]
        """
        return self.kwargs['hidden']

    @property
    def queries(self) -> Optional[List]:
        """
        It's a property used to get all queries of the DropMenu

        :returns: queries: List (or) None
        """

        return self.kwargs['queries']

    def after_response(self, **options):
        """
        It's an event type function which changes the provided options after option (on-select) of the DropMenu

        :param options: takes variables
        :returns: None
        """

        kwargs = {}
        if len(options) >= 1:
            for key in options:
                if key not in self.kwargs.keys():
                    raise ValueError(f"Invalid option `--{key}`")
                else:
                    kwargs.update({key: options[key]})

            self.after_ = kwargs

    @property
    def after_resp(self) -> Optional[Dict]:
        """
        It's a property used to get all options which are to be changed after option (on-select) of the DropMenu

        :returns: Dict (or) None
        """

        return self.after_

    async def add_coro_func(self, function, *args, **kwargs):
        """
        It's an asynchronous function which stores same function type
        and adds the func to DropMenu for execution after getting an option selected

        :param function: takes asynchronous function
        :param args: takes args of the function provided by a user
        :param kwargs: takes kwargs of the function provided by a user
        :returns: None
        """

        self.kwargs['coro_func'] = lambda: function(*args, **kwargs)

    def add_func(self, function, *args, **kwargs):
        """
        It's a function which stores same function type and
        adds the func to DropMenu for execution after getting an option selected

        :param function: takes a function
        :param args: takes args of the function provided by a user
        :param kwargs: takes kwargs of the function provided by a user
        :returns: None
        """

        self.kwargs['func'] = lambda: function(*args, **kwargs)

    def add_query(self, *query: Tuple[str, Union[str, discord.Embed]]):
        """
        It's an event type function used to add queries for the DropMenu

        :param query: takes option-name and response
        :returns: None
        """

        for query in query:
            queries_: list = self.kwargs['queries']
            queries_.append(query)

    def add_queries(self, *queries: Tuple[List[str], Union[str, discord.Embed]]):
        """
        It's an event type function used to add queries for the DropMenu

        :param queries: takes option-name and response
        :returns: None
        """

        for query_ in queries:
            queries_: list = self.kwargs['queries']
            queries_.append(query_)

    @staticmethod
    def convert_resp(content: str, values: list):
        """
        It's not a reusable function

        Used for formatting the provided content with respective values

        :param content: takes content/ message
        :param values: selected values of the DropMenu
        :returns: None
        """

        count = 1
        max_count = len(values) + 5
        while count <= max_count:
            if "{values}" in content:
                try:
                    content = content.replace("{values}", ", ".join(values))
                except ValueError:
                    raise ValueError("Format key not Found\nTry this formatters: `{values}`, `{{values[index]}}`")
            elif "{{values[" in content:
                try:
                    x = content.index("{{")
                    y = content.index("}}")

                    index: int = int(content[content.index("[") + 1: content.index("]")])
                    content = content.replace(f"{content[x: y + 2]}", values[index])
                except ValueError:
                    raise ValueError("Format key not Found\nTry this formatters: `{values}`, `{{values[index]}}`")
                except IndexError:
                    raise IndexError(f"list index out of range (len(values) = {len(values)})")
            count += 1

        return content


class Menu(ui.Select):
    def __init__(self, root: ClassVar, menu: SDropMenu):
        self.root = root
        self.menu = menu
        self.menu_args = menu.args
        super().__init__(
            custom_id=self.menu_args['custom_id'], placeholder=self.menu_args['placeholder'],
            min_values=self.menu_args['min_values'], max_values=self.menu_args['max_values'],
            options=self.menu_args['options'], disabled=self.menu_args['disabled'], row=self.menu_args['row']
        )

    def query_conditions(self, queries) -> Union[str, discord.Embed, None]:
        values_ = self.values
        for val, response_ in queries:
            if isinstance(val, list):
                if sorted(val) == sorted(values_):
                    return response_
            else:
                for value_ in values_:
                    if val == value_:
                        return response_

        return None

    async def callback(self, interaction: discord.Interaction):
        self.menu.interaction = interaction
        checked = check_for_Invoker(self.menu, interaction)
        if checked:
            self.menu.selected_values = self.values

            if self.menu_args['coro_func'] is not None:
                func = self.menu_args['coro_func']
                await func()

            if self.menu_args['func'] is not None:
                func = self.menu_args['func']
                func()

            if self.menu_args['response'] is None:
                resp = SDropMenu.convert_resp("Options: ' {values} ' has been selected !", self.values)
            else:
                resp = self.menu_args['response']

            if self.menu.after_resp is not None:
                for key in self.menu.after_resp:
                    self.menu.update_one(self.menu.after_resp[key], key)

            if len(self.menu.queries) >= 1:
                get_queries = self.query_conditions(self.menu.queries)
            else:
                get_queries = None

            emph_ = self.menu.is_ephemeral

            if get_queries is not None:
                resp_ = get_queries
            else:
                resp_ = None

            menu_ = self.root()
            view_ = menu_.view()

            if self.menu_args['rewrite']:
                if resp_ is not None:
                    if is_embed(resp_):
                        resp_.description = SDropMenu.convert_resp(resp_.description, self.values)
                        await interaction.message.edit(content=' ', embed=resp_, view=view_)
                    else:
                        resp_ = SDropMenu.convert_resp(resp_, self.values)
                        await interaction.message.edit(content=resp_, embed=None, view=view_)
                else:
                    if is_embed(resp):
                        resp.description = SDropMenu.convert_resp(resp.description, self.values)
                        await interaction.message.edit(content=' ', embed=resp, view=view_)
                    else:
                        resp = SDropMenu.convert_resp(resp, self.values)
                        await interaction.message.edit(content=resp, embed=None, view=view_)
            else:
                await interaction.message.edit(view=view_)
                if resp_ is not None:
                    if is_embed(resp_):
                        resp_.description = SDropMenu.convert_resp(resp_.description, self.values)
                        await interaction.response.send_message(embed=resp_, ephemeral=emph_)
                    else:
                        resp_ = SDropMenu.convert_resp(resp_, self.values)
                        await interaction.response.send_message(content=resp_, ephemeral=emph_)
                else:
                    if is_embed(resp):
                        resp.description = SDropMenu.convert_resp(resp.description, self.values)
                        await interaction.response.send_message(content=' ', embed=resp, ephemeral=emph_)
                    else:
                        resp = SDropMenu.convert_resp(resp, self.values)
                        await interaction.response.send_message(content=resp, ephemeral=emph_)



import discord
from typing import *
from discord import utils

MISSING = utils.MISSING
DEFAULT_TIMEOUT: Union[int, float] = 180.0


def is_embed(response: Any) -> bool:
    """
    Checks whether the type of response is str (or) embed

    :param response: takes str (or) embed
    :return: bool, i.e [True, False]
    """
    return isinstance(response, discord.Embed)


def check_for_Invoker(component, interaction: discord.Interaction) -> bool:
    """
    Checks whether the author of the component and interaction are same or not

    :param component: Takes either button: SButton or dropmenu: SDropMenu
    :param interaction: Takes interaction type discord.Interaction
    :return: bool, i.e [True, False]
    """

    args_ = component.args
    type_ = args_['verify']
    if type_:
        if interaction.user == component.author:
            return True
        else:
            return False
    else:
        return True



from btns_menus.builds.button_ import Btn, SButton, DEFAULT_TIMEOUT

from typing import *
import discord
from discord import ui


class SingleButton:
    def __init__(self, author: discord.Member, button1: SButton, /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Button

        :param author: Interaction User
        :param button1: Takes Button Type SButton
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btn1 = button1

        if self.btn1.author is None:
            self.btn1.update(author=self.author)

        self.root_ = lambda: SingleButton(
            self.author, self.btn1, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        if not self.btn1.hidden:
            view_.add_item(Btn(self.root_, self.btn1))

        return view_


class DuoButton:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons

        :param author: Interaction User
        :param button1: Takes Button Type SButton
        :param button2: Takes Button Type SButton
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btns = [button1, button2]
        for btn_ in self.btns:
            if btn_.author is None:
                btn_.update(author=self.author)

        self.root_ = lambda: DuoButton(
            self.author, self.btns[0], self.btns[1], timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for btn_ in self.btns:
            if not btn_.hidden:
                view_.add_item(Btn(self.root_, btn_))

        return view_


class TrioButton:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton, button3: SButton,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons

        :param author: Interaction User
        :param button1: Takes Button Type SButton
        :param button2: Takes Button Type SButton
        :param button3: Takes Button Type SButton
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """

        self.author = author
        self.timeout = timeout

        self.btns = [button1, button2, button3]
        for btn_ in self.btns:
            if btn_.author is None:
                btn_.update(author=self.author)

        self.root_ = lambda: TrioButton(
            self.author, self.btns[0], self.btns[1], self.btns[2], timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for btn_ in self.btns:
            if not btn_.hidden:
                view_.add_item(Btn(self.root_, btn_))

        return view_


class QuartetButton:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton, button3: SButton, button4: SButton,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons

        :param author: Interaction User
        :param button1: Takes Button Type SButton
        :param button2: Takes Button Type SButton
        :param button3: Takes Button Type SButton
        :param button4: Takes Button Type SButton
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """
        self.author = author
        self.timeout = timeout

        self.btns = [button1, button2, button3, button4]
        for btn_ in self.btns:
            if btn_.author is None:
                btn_.update(author=self.author)

        self.root_ = lambda: QuartetButton(self.author, self.btns[0], self.btns[1],
                                           self.btns[2], self.btns[3], timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for btn_ in self.btns:
            if not btn_.hidden:
                view_.add_item(Btn(self.root_, btn_))

        return view_


class QuintetButton:
    def __init__(self, author: discord.Member, button1: SButton, button2: SButton,
                 button3: SButton, button4: SButton, button5: SButton,
                 /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons

        :param author: Interaction User
        :param button1: Takes Button Type SButton
        :param button2: Takes Button Type SButton
        :param button3: Takes Button Type SButton
        :param button4: Takes Button Type SButton
        :param button5: Takes Button Type SButton
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """
        self.author = author
        self.timeout = timeout

        self.btns = [button1, button2, button3, button4, button5]
        for btn_ in self.btns:
            if btn_.author is None:
                btn_.update(author=self.author)

        self.root_ = lambda: QuintetButton(self.author, self.btns[0], self.btns[1], self.btns[2],
                                           self.btns[3], self.btns[4], timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for btn_ in self.btns:
            if not btn_.hidden:
                view_.add_item(Btn(self.root_, btn_))

        return view_


class MultiButton:
    def __init__(self, author: discord.Member, buttons: List[SButton], /, timeout: Optional[float] = DEFAULT_TIMEOUT):
        """
        Responsive Buttons

        :param author: Interaction User
        :param buttons: Takes Button Type SButton
        :param timeout: Interaction Timeout

        :returns: view: discord.ui.View
        """
        self.author = author
        self.timeout = timeout
        self.btns = buttons

        for btn_ in self.btns:
            if btn_.author is None:
                btn_.update(author=self.author)

        self.root_ = lambda: MultiButton(
            self.author, self.btns, timeout=self.timeout)

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = ui.View(timeout=self.timeout)
        for btn_ in self.btns:
            if not btn_.hidden:
                view_.add_item(Btn(self.root_, btn_))

        return view_



from btns_menus.Combinations import *

import discord
from discord import ButtonStyle, SelectOption


def SOption(*, name: str, embed_: discord.Embed, description: str = None,
            emoji: Union[str, discord.Emoji, discord.PartialEmoji] = None
            ) -> Dict:
    """It's a decorator used to overwrite options in discord.ui.Select

    :param name: label for option
    :param embed_: If option is selected, the embed will be sent !
    :param description: Description for the option
    :param emoji: Emoji for the option

    :returns: Dict
    """
    decorator_ = {
        "name": name, "description": description, "embed": embed_, "emoji": emoji
    }
    return decorator_


class Paginator:
    def __init__(self,
                 author: discord.Member,
                 embeds: List[discord.Embed],
                 *,
                 commands_list: List[SOption] = None,
                 buttons: List[SButton] = None,
                 menus: List[SDropMenu] = None,
                 footer: str = None,
                 timeout: Optional[float] = DEFAULT_TIMEOUT
                 ):
        """
        Paginator is used to show users the given embeds in pages format using **navigation Buttons and DropMenus**

        :param author: User who will interact with the Paginator
        :param embeds: The list of embeds that acts as Pages
        :param commands_list: The list of options which are shown as options in a Drop Menu
        :param buttons: List of buttons used to navigate of interact with the pages/embeds
        :param menus: List of Menus used to make a user select options from it
        :param footer: Used to add content into text in footer of discord.Embed
        :param timeout: Timeout of the interaction

        :returns: view: discord.ui.View
        """

        self.author = author
        self.embeds = embeds
        self.cmds_list = commands_list
        self.buttons = [] if buttons is None else buttons
        self.menus = [] if menus is None else menus
        self.timeout = timeout

        self.pages: int = 0

        def footer_for_PG(content: str = None) -> Optional[str]:
            if content is not None:
                conv_content = content.replace("{user}", str(self.author))
                conv_content = conv_content.replace("{username}", self.author.display_name)
                conv_content = conv_content.replace("{pg}", str(self.pages + 1))
                conv_content = conv_content.replace("{total_pgs}", str(len(self.embeds)))
                conv_content = conv_content.replace("{guild}", self.author.guild.name)

                return conv_content
            else:
                return None

        self.footer_for_PG = footer_for_PG
        self.footer = footer

        self.index_for_home = 0
        self.index_for_forward = 1
        self.index_for_backward = 2
        self.index_for_delete = 3
        self.index_for_cmds = 0

        self.home_btn: Optional[SButton] = None
        self.forward_btn: Optional[SButton] = None
        self.backward_btn: Optional[SButton] = None
        self.delete_menu: Optional[SButton] = None
        self.cmds_menu: Optional[SDropMenu] = None

        self.create_pages()

    def create_pages(self):
        buttons_ = []
        for button_ in self.buttons:
            if button_.id is not None:
                if button_.id.lower() == "home":
                    self.home_btn = button_
                    self.index_for_home = self.buttons.index(button_)
                    buttons_.append(button_)
                if button_.id.lower() == "forward":
                    self.forward_btn = button_
                    self.index_for_forward = self.buttons.index(button_)
                    buttons_.append(button_)
                if button_.id.lower() == "backward":
                    self.backward_btn = button_
                    self.index_for_backward = self.buttons.index(button_)
                    buttons_.append(button_)
                if button_.id.lower() == "delete":
                    self.delete_menu = button_
                    self.index_for_delete = self.buttons.index(button_)
                    buttons_.append(button_)

        for btn in buttons_:
            self.buttons.remove(btn)

        for menu_ in self.menus:
            if menu_.id is not None:
                if menu_.id.lower() in ["commands-list", "cmds-list"]:
                    self.cmds_menu = menu_
                    self.index_for_cmds = self.menus.index(menu_)
                    self.menus.remove(menu_)

        self.home_btn = SButton(label="Home", emoji="🏠", response=self.embeds[0], style=ButtonStyle.green,
                                rewrite=True, disabled=True) if self.home_btn is None else self.home_btn
        self.forward_btn = SButton(label='', emoji="⏩", rewrite=True,
                                   style=ButtonStyle.secondary) if self.forward_btn is None else self.forward_btn
        self.backward_btn = SButton(label='', emoji="⏪", style=ButtonStyle.secondary,
                                    rewrite=True) if self.backward_btn is None else self.backward_btn
        self.delete_menu = SButton(label='Delete Menu', emoji="🗑️", style=ButtonStyle.danger,
                                   delete_msg=True) if self.delete_menu is None else self.delete_menu
        self.cmds_menu = SDropMenu(placeholder="Select any one Module",
                                   rewrite=True) if self.cmds_menu is None else self.cmds_menu

        if self.home_btn.kwargs['response'] is None:
            em = self.embeds[0]
            if len(em.footer.text) == 0 and self.footer is not None:
                em.set_footer(text=self.footer_for_PG(self.footer))
            self.home_btn.update(response=em, rewrite=True)

        self.build_pages()

    def build_pages(self):
        def home_page():
            self.pages = 0

            self.home_btn.update(disabled=True)
            self.forward_btn.update(disabled=False)
            self.backward_btn.update(disabled=False)

        def forward_pages():
            self.pages += 1
            if self.pages >= len(self.embeds) - 1:
                self.pages = len(self.embeds) - 1
                self.forward_btn.update(disabled=True)

            self.backward_btn.update(disabled=False)
            self.home_btn.update(disabled=False)

            em = self.embeds[self.pages]
            if len(em.footer.text) == 0 and self.footer is not None:
                em.set_footer(text=self.footer_for_PG(self.footer))
            self.forward_btn.update_one(em, "response")

        def backward_pages():
            self.pages -= 1
            if self.pages <= 0:
                self.pages = 0

                self.home_btn.update(disabled=True)
                self.backward_btn.update(disabled=True)
            else:
                self.backward_btn.update(disabled=False)
                self.home_btn.update(disabled=False)

            self.forward_btn.update(disabled=False)

            em = self.embeds[self.pages]
            if len(em.footer.text) == 0 and self.footer is not None:
                em.set_footer(text=self.footer_for_PG(self.footer))
            self.backward_btn.update_one(em, "response")

        if self.cmds_list is not None:
            options = []
            for option in self.cmds_list:
                opt_ = SelectOption(
                    label=option['name'], description=option['description'], emoji=option['emoji'])
                options.append(opt_)

            self.cmds_menu.update(options=options)
            for option in self.cmds_list:
                self.cmds_menu.add_query((option['name'], option['embed']))

            if self.cmds_menu.kwargs['func'] is None and self.cmds_menu.kwargs['coro_func'] is None:
                def reset_view():
                    self.home_btn.update(disabled=False)
                    self.forward_btn.update(disabled=True)
                    self.backward_btn.update(disabled=True)

                self.cmds_menu.add_func(reset_view)

            self.menus.insert(self.index_for_cmds, self.cmds_menu)

        if self.home_btn.kwargs['func'] is None and self.home_btn.kwargs['coro_func'] is None:
            self.home_btn.add_func(home_page)
        if self.forward_btn.kwargs['func'] is None and self.forward_btn.kwargs['coro_func'] is None:
            self.forward_btn.add_func(forward_pages)
        if self.backward_btn.kwargs['func'] is None and self.backward_btn.kwargs['coro_func'] is None:
            self.backward_btn.add_func(backward_pages)

        created_btns = [self.home_btn, self.forward_btn,
                        self.backward_btn, self.delete_menu]
        stored_indexes = [self.index_for_home, self.index_for_forward,
                          self.index_for_backward, self.index_for_delete]
        for x in range(len(created_btns)):
            self.buttons.insert(stored_indexes[x], created_btns[x])

    def view(self) -> ui.View:
        """:returns: discord.ui.View"""

        view_ = MultiBtnAndMenu(self.author, self.buttons,
                                self.menus, timeout=self.timeout).view()
        return view_



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="discord-btns-menus",
    version="0.2.2b",
    author="P. Sai Keerthan Reddy",
    author_email="saikeerthan.keerthan.9@gmail.com",
    description="A responsive package for Buttons, DropMenus, Combinations and Paginator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Modern-Realm/discord_btns_menus",
    project_urls={
        "Documentation": "https://modern-realm.github.io/discord_btns_menus/",
        "Bug Tracker": "https://github.com/Modern-Realm/discord_btns_menus/issues",
        "Examples": "https://github.com/Modern-Realm/discord_btns_menus/tree/main/Examples",
        "Source": "https://github.com/Modern-Realm/discord_btns_menus/tree/main/package/btns_menus",
        "Discord Server": "https://discord.gg/GVMWx5EaAN",
    },
    keywords=["py-cord", "pycord", "buttons", "dropmenus", "menus",
              "nextcord", "disnake", "discordpy", "pycord-btns-menus"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Topic :: Internet",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
        "Typing :: Typed",
    ],
    license="MIT",
    package_dir={"": "package"},
    packages=["btns_menus", "btns_menus.builds"],
    include_package_data=True,
    python_requires=">=3.8",
)



"""
Name: pycord-btns-menus

Version: 0.1.7

Summary: A responsive package for Buttons, DropMenus and Combinations

Home-page: https://github.com/skrphenix/pycord_btns_menus

Author: P. Sai Keerthan Reddy

Author-email: saikeerthan.keerthan.9@gmail.com

License: MIT

Location: c:/users/sai keerthan/appdata/local/programs/python/python39/lib/site-packages

Requires: py-cord
"""
